(* {{{ COPYING *)
(*
 * +---------------------------------------------------------------+
 * | Copyright (C) 2006 Florent Monnier                            |
 * +---------------------------------------------------------------+
 * | This is a small quick and dirty starting point for X demos.   |
 * +---------------------------------------------------------------+
 * |                                                               |
 * | This program is free software; you can redistribute it and/or |
 * | modify it under the terms of the GNU General Public License   |
 * | as published by the Free Software Foundation; either version  |
 * | 2 of the License, or (at your option) any later version.      |
 * |                                                               |
 * | This program is distributed in the hope that it will be       |
 * | useful, but WITHOUT ANY WARRANTY; without even the implied    |
 * | warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       |
 * | PURPOSE. See the GNU General Public License for more details. |
 * |                                                               |
 * | http://www.fsf.org/licensing/licenses/gpl.html                |
 * |                                                               |
 * | You should have received a copy of the GNU General Public     |
 * | License along with this program; if not,                      |
 * | write to the Free Software Foundation, Inc.,                  |
 * | 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA  |
 * |                                                               |
 * +---------------------------------------------------------------+
 * | Author: Florent Monnier <fmonnier@linux-nantes.fr.eu.org>     |
 * +---------------------------------------------------------------+
 *
 * }}} *)

(* {{{ init vars *)

let width, height = 1024, 768 ;;
let total_seconds  = 192 ;;
let frames_per_sec = 64 ;;

(* }}} *)

(* {{{ utilities *)

type init_item = Width | Height | Frames_per_sec | Total_seconds ;;

let init item =
  match item with
  | Width -> width
  | Height -> height
  | Frames_per_sec -> frames_per_sec
  | Total_seconds -> total_seconds
;;

external set_item: init_item -> int -> unit = "ml_set_item";;


type graphic_context = Background | Main_Foreground | Foreground_i of int

external xDrawLine_background:      x1:int -> y1:int -> x2:int -> y2:int -> unit = "ml_xdrawline_background";;
external xDrawLine_main_foreground: x1:int -> y1:int -> x2:int -> y2:int -> unit = "ml_xdrawline_main_foreground";;
external xDrawLine_foreground_i: index:int -> x1:int -> y1:int -> x2:int -> y2:int -> unit = "ml_xdrawline_foreground_i";;

external xDrawRectangle_background:      x:int -> y:int -> width:int -> height:int -> unit = "ml_xdrawrectangle_background";;
external xDrawRectangle_main_foreground: x:int -> y:int -> width:int -> height:int -> unit = "ml_xdrawrectangle_main_foreground";;
external xDrawRectangle_foreground_i: index:int -> x:int -> y:int -> width:int -> height:int -> unit = "ml_xdrawrectangle_foreground_i";;

external xSet_foreground_i: index:int -> red:int -> green:int -> blue:int -> unit = "ml_xsetforeground";;
let xSet_main_foreground ~red ~green ~blue = xSet_foreground_i ~index:0 ~red ~green ~blue;;

let xDrawLine ~color ~x1 ~y1  ~x2 ~y2 =
  match color with
  | Background         -> xDrawLine_background      ~x1 ~y1  ~x2 ~y2;
  | Main_Foreground    -> xDrawLine_main_foreground ~x1 ~y1  ~x2 ~y2;
  | Foreground_i index -> xDrawLine_foreground_i ~index ~x1 ~y1 ~x2 ~y2;
;;

let xDrawRectangle ~color ~x ~y  ~width ~height =
  if width  < 1 then invalid_arg "xDrawRectangle: width should be positive";
  if height < 1 then invalid_arg "xDrawRectangle: height should be positive";
  match color with
  | Background         -> xDrawRectangle_background      ~x ~y  ~width ~height;
  | Main_Foreground    -> xDrawRectangle_main_foreground ~x ~y  ~width ~height;
  | Foreground_i index -> xDrawRectangle_foreground_i ~index ~x ~y  ~width ~height;
;;

let xDrawRectangle' ~color ~x1 ~y1  ~x2 ~y2 =
  let (x, width)  = if x1 < x2 then (x1, x2 - x1) else (x2, x1 - x2)
  and (y, height) = if y1 < y2 then (y1, y2 - y1) else (y2, y1 - y2)
  in
  match color with
  | Background         -> xDrawRectangle_background      ~x ~y  ~width ~height;
  | Main_Foreground    -> xDrawRectangle_main_foreground ~x ~y  ~width ~height;
  | Foreground_i index -> xDrawRectangle_foreground_i ~index ~x ~y  ~width ~height;
;;


external xDrawPoint_main_foreground: x:int -> y:int -> unit = "ml_xdrawpoint_main_foreground";;
external xDrawPoint_foreground_i: index:int -> x:int -> y:int -> unit = "ml_xdrawpoint_foreground_i";;


type points_list = (int * int) list ;;

external _xDrawPoints_main_foreground: points_list:points_list -> npoints:int -> unit = "ml_xdrawpoints_main_foreground"
let xDrawPoints_main_foreground ~points =
  let npoints = List.length points in
  _xDrawPoints_main_foreground ~points_list:points ~npoints
;;

external _xDrawPoints_foreground_i: index:int -> points_list:points_list -> npoints:int -> unit = "ml_xdrawpoints_foreground_i"
let xDrawPoints_foreground_i ~index ~points =
  let npoints = List.length points in
  _xDrawPoints_foreground_i ~index ~points_list:points ~npoints
;;


(* }}} *)

(* {{{ iteration loop *)

(* {{{ class line *)

Random.self_init();;

class line ~color =
  object(self)
    val mutable vect_coords = ( 0,2, 2,0 )
    val mutable line_coords =
      ( Random.int width, Random.int height,
        Random.int width, Random.int height )

    method step =
      let (x1,y1, x2,y2) = line_coords
      and (vect_x1, vect_y1,
           vect_x2, vect_y2) = vect_coords
      in
      let (x1,y1, x2,y2) =
        ( x1 + vect_x1,
          y1 + vect_y1,
          x2 + vect_x2,
          y2 + vect_y2 )
      in
      let (x1, y1) = if y1 > height || y1 < 0 then (self#y_overflow y1) else (x1, y1)
      and (x2, y2) = if x2 > width  || x2 < 0 then (self#x_overflow x2) else (x2, y2)
      in
      line_coords <- (x1,y1, x2,y2);

    method private y_overflow y = (self#random_x, self#clip_y y)
    method private x_overflow x = (self#clip_x x, self#random_y)

    method private random_x = Random.int width
    method private random_y = Random.int height

    method private _clip_x x = abs(x mod width)
    method private _clip_y y = abs(y mod height)

    method private clip_x x =
      let (x1,y1, x2,y2) = vect_coords in
      let x2 = - x2 in
      vect_coords <- (x1,y1, x2,y2);
      x

    method private clip_y y =
      let (x1,y1, x2,y2) = vect_coords in
      let y1 = - y1 in
      vect_coords <- (x1,y1, x2,y2);
      y

    method draw =
      let (x1,y1, x2,y2) = line_coords in
      xDrawLine ~color ~x1 ~y1 ~x2 ~y2;
  end
;;

(* }}} *)
(* {{{ create lines list *)

let color_lines_number = 6 ;;
let black_lines_number = 3 ;;


let rec make_lines_list ~color acc i =
  if i <= 0 then acc
  else make_lines_list ~color ((new line ~color)::acc) (pred i)
;;

let lines =
  let white_lines =
    make_lines_list Main_Foreground [] color_lines_number
  in
  make_lines_list Background white_lines  black_lines_number;
;;

(* }}} *)

type color = { r:int; g:int; b:int };;

let color_a = {r=15000; g=25000; b=0};;
let color_b = {r=30000; g=200;   b=6000};;

let main_loop  i =
  (* {{{ change foreground color *)

  if false then
    begin
      if (succ i mod 600) = 0 then
        xSet_foreground_i ~index:0 ~red:30000 ~green:200 ~blue:6000;

      if ((i+301) mod 600) = 0 then
        xSet_foreground_i ~index:0 ~red:15000 ~green:25000 ~blue:0;
    end
  else
    begin
      let percent = float(i mod 600) /. 600.0 in
      let percent_inv = 1.0 -. percent in
      let red   = truncate(percent *. float color_a.r +. percent_inv *. float color_b.r)
      and green = truncate(percent *. float color_a.g +. percent_inv *. float color_b.g)
      and blue  = truncate(percent *. float color_a.b +. percent_inv *. float color_b.b)
      in
      xSet_foreground_i ~index:0 ~red ~green ~blue;
    end;

  (* }}} *)

  (* evolution of the lines *)
  List.iter (fun line -> line#step; line#draw) lines;

  (* {{{ rectangle eraser *)

  let draw_rectangle ~color i =
    let (x, y) = (i mod (width/2), i mod (height/2)) in
    let (_width, _height) = ((pred width) - 2 * x, (pred height) - 2 * y) in
    xDrawRectangle ~color ~x ~y  ~width:_width ~height:_height;
  in
  draw_rectangle Background i;
  draw_rectangle Main_Foreground (succ i);
  (* }}} *)
;;

(* }}} *)

(* {{{ main *)

let _ =
  Callback.register "ocaml init" init;
  Callback.register "ocaml main loop" main_loop;
;;

(* }}} *)


(* vim: sw=2 sts=2 ts=2 et fdm=marker
 *)
