
(** A quick and dirty starting point to make X demo. *)

val width: int ;;
val height: int ;;
(** the width and height of the window. *)
val total_seconds: int;;
(** the number of seconds of the demo. *)
val frames_per_sec: int ;;
(** the number of frames per seconds. *)

(** by default there are only two colors: the background which default color is
    black, and the main foreground which you can change the color with
    [xSet_main_foreground].

    If you wish more foregrounds edit {i main.c} and change the value defined for the macro
    [FOREGROUNDS]. Then you will be able to use Foreground_i index which max
    value is the number of foregrounds you have defined - 1.
    (the index 0 is the main foreground) *)
type graphic_context = Background | Main_Foreground | Foreground_i of int

val xDrawLine_main_foreground: x1:int -> y1:int -> x2:int -> y2:int -> unit
(** Draw a line with the main foreground color.
    Set this color with [xSet_main_foreground].
    x1,y1 is the first point, and x2,y2 the second one.
    0,0 is the upper left corner, and (pred width, pred height) the lower right one. *)
val xDrawLine_background: x1:int -> y1:int -> x2:int -> y2:int -> unit
(** draw a line with the background color. *)
val xDrawLine_foreground_i: index:int -> x1:int -> y1:int -> x2:int -> y2:int -> unit
(** draw a line with the foreground color of the given index.
    This index have to be between 0 (inclusive) and [FOREGROUNDS] exclusive.
    The foreground index 0 is the main foreground. [FOREGROUNDS] is defined in {i main.c}. *)
val xDrawLine: color:graphic_context -> x1:int -> y1:int -> x2:int -> y2:int -> unit
(** a generic draw line with which one can switch with the [graphic_context] type. *)

val xSet_main_foreground: red:int -> green:int -> blue:int -> unit
(** change the color of the main foreground. back is 0,0,0 and white is 65535,65535,65535 *)
val xSet_foreground_i: index:int -> red:int -> green:int -> blue:int -> unit
(** change the color of the given index foreground. *)

val xDrawRectangle_main_foreground: x:int -> y:int -> width:int -> height:int -> unit
(** draw a rectangle. *)
val xDrawRectangle_background:      x:int -> y:int -> width:int -> height:int -> unit
val xDrawRectangle_foreground_i: index:int -> x:int -> y:int -> width:int -> height:int -> unit

val xDrawRectangle : color:graphic_context -> x:int -> y:int -> width:int -> height:int -> unit
(** draw a rectangle. x,y is the point of the upper-left corner. width and
    height have to be greater than 1. *)
val xDrawRectangle': color:graphic_context -> x1:int -> y1:int -> x2:int -> y2:int -> unit
(** draw a rectangle. x1,y1 is one corner, x2,y2 is an opposite corner.  *)


val xDrawPoint_main_foreground: x:int -> y:int -> unit
(** draw a point *)
val xDrawPoint_foreground_i: index:int -> x:int -> y:int -> unit


type points_list = (int * int) list ;;

val xDrawPoints_main_foreground: points_list:points_list -> unit
(** draw points. This function will give better performence to draw a great
    number of points than an iteration over [xDrawPoint_main_foreground]. *)

val xDrawPoints_foreground_i: index:int -> points_list:points_list -> unit
(** draw points with the given foreground. *)


