/* {{{ COPYING 
 *
 * +---------------------------------------------------------------+
 * | Copyright (C) 2006 Florent Monnier                            |
 * +---------------------------------------------------------------+
 * | This is a small quick and dirty starting point for X demos.   |
 * +---------------------------------------------------------------+
 * |                                                               |
 * | This program is free software; you can redistribute it and/or |
 * | modify it under the terms of the GNU General Public License   |
 * | as published by the Free Software Foundation; either version  |
 * | 2 of the License, or (at your option) any later version.      |
 * |                                                               |
 * | This program is distributed in the hope that it will be       |
 * | useful, but WITHOUT ANY WARRANTY; without even the implied    |
 * | warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       |
 * | PURPOSE. See the GNU General Public License for more details. |
 * |                                                               |
 * | http://www.fsf.org/licensing/licenses/gpl.html                |
 * |                                                               |
 * | You should have received a copy of the GNU General Public     |
 * | License along with this program; if not,                      |
 * | write to the Free Software Foundation, Inc.,                  |
 * | 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA  |
 * |                                                               |
 * +---------------------------------------------------------------+
 * | Author: Florent Monnier <fmonnier@linux-nantes.fr.eu.org>     |
 * +---------------------------------------------------------------+
 *
 * }}} */
/* {{{ headers */

#include <stdio.h>
#include <stdlib.h>

#include <caml/mlvalues.h>
#include <caml/callback.h>
#include <caml/alloc.h>
#include <caml/memory.h>
#include <caml/fail.h>
#include <caml/custom.h>
#include <caml/intext.h>
/* {{{
}}} */
#include <X11/Xlib.h>
#include <assert.h>
#include <unistd.h>

/* }}} */
/* {{{ init vars */

#define NIL (0)

Display *dpy;
Window w;
Colormap colormap;
#define FOREGROUNDS 1
GC gc_foreground[FOREGROUNDS];
GC gc_background;

int width  = 800;
int height = 600;
int frames_per_sec = 32;
int total_seconds  = 128;

/* }}} */

/* {{{ callbacks */

void call_caml_loop_iter(int arg)
{
    static value * closure_loop = NULL;
    if (closure_loop == NULL) {
        /* First time around, look up by name */
        closure_loop = caml_named_value("ocaml main loop");
    }
    caml_callback(*closure_loop, Val_int(arg));
}

int call_caml_init(int arg)
{
    CAMLparam0();
    CAMLlocal1(init_val);

    static value * closure_init = NULL;
    if (closure_init == NULL) {
        /* First time around, look up by name */
        closure_init = caml_named_value("ocaml init");
    }
    init_val = caml_callback(*closure_init, Val_int(arg));

    CAMLreturn( Int_val(init_val) );
}
/* }}} */

/* {{{ XDrawLine() */

void ml_xdrawline_foreground_i( value i, value x1, value y1, value x2, value y2 )
{
    XDrawLine(dpy, w, gc_foreground[Int_val(i)],
            Int_val(x1),
            Int_val(y1),
            Int_val(x2),
            Int_val(y2) );
}

void ml_xdrawline_main_foreground( value x1, value y1, value x2, value y2 )
{
    XDrawLine(dpy, w, gc_foreground[0],
            Int_val(x1),
            Int_val(y1),
            Int_val(x2),
            Int_val(y2) );
}

void ml_xdrawline_background( value x1, value y1, value x2, value y2 )
{
    XDrawLine(dpy, w, gc_background,
            Int_val(x1),
            Int_val(y1),
            Int_val(x2),
            Int_val(y2) );
}

/* }}} */
/* {{{ XSetForeground() */

void ml_xsetforeground( value i, value red, value green, value blue )
{
    XColor system_color;
    system_color.red   = Int_val(red);
    system_color.green = Int_val(green);
    system_color.blue  = Int_val(blue);
    Status rc = XAllocColor(dpy, colormap, &system_color);
    // make sure the allocation succeeded.
    if (rc == 0) {
        fprintf(stderr, "XAllocColor - allocation of RGB color failed.\n");
    } else {
        // do something with the allocated color...
        XSetForeground(dpy, gc_foreground[Int_val(i)], system_color.pixel);
    }
}

/* }}} */
/* {{{ XDrawPoint() */

void ml_xdrawpoint_foreground_i( value i, value x, value y )
{
    XDrawPoint(dpy, w, gc_foreground[Int_val(i)],
            Int_val(x),
            Int_val(y) );
}

void ml_xdrawpoint_main_foreground( value x, value y )
{
    XDrawPoint(dpy, w, gc_foreground[0],
            Int_val(x),
            Int_val(y) );
}

/* }}} */
/* {{{ XDrawPoints() */

CAMLprim value
ml_xdrawpoints_main_foreground( value points_list, value npoints )
{
    CAMLparam2(points_list, npoints);
    CAMLlocal1(ml_point);

    int _npoints = Int_val(npoints);

    /* typedef struct { short x, y; } XPoint; */
    XPoint *points;
    points = malloc(_npoints * sizeof(XPoint));

    XPoint _point;
    int p;
    for (p=0; p < _npoints; p++) {
        ml_point = Field(points_list,0);
        _point.x = Int_val(Field(ml_point,0));
        _point.y = Int_val(Field(ml_point,1));
        points[p] = _point;
        points_list = Field(points_list,1);  /* tail */
    }

    XDrawPoints(dpy, w, gc_foreground[0], points, _npoints, CoordModeOrigin);

    free(points);
    CAMLreturn(Val_unit);
}


CAMLprim value
ml_xdrawpoints_foreground_i( value i, value points_list, value npoints )
{
    CAMLparam2(points_list, npoints);
    CAMLlocal1(ml_point);

    int _npoints = Int_val(npoints);

    /* typedef struct { short x, y; } XPoint; */
    XPoint *points;
    points = malloc(_npoints * sizeof(XPoint));

    XPoint _point;
    int p;
    for (p=0; p < _npoints; p++) {
        ml_point = Field(points_list,0);
        _point.x = Int_val(Field(ml_point,0));
        _point.y = Int_val(Field(ml_point,1));
        points[p] = _point;
        points_list = Field(points_list,1);  /* tail */
    }

    XDrawPoints(dpy, w, gc_foreground[Int_val(i)], points, _npoints, CoordModeOrigin);

    free(points);
    CAMLreturn(Val_unit);
}

/* }}} */
/* {{{ XDrawRectangle() */

void ml_xdrawrectangle_foreground_i( value i, value x, value y, value _width, value _height )
{
    XDrawRectangle(dpy, w, gc_foreground[Int_val(i)],
            Int_val(x),
            Int_val(y),
            Int_val(_width),
            Int_val(_height) );
}

void ml_xdrawrectangle_main_foreground( value x, value y, value _width, value _height )
{
    XDrawRectangle(dpy, w, gc_foreground[0],
            Int_val(x),
            Int_val(y),
            Int_val(_width),
            Int_val(_height) );
}

void ml_xdrawrectangle_background( value x, value y, value _width, value _height )
{
    XDrawRectangle(dpy, w, gc_background,
            Int_val(x),
            Int_val(y),
            Int_val(_width),
            Int_val(_height) );
}
/* }}} */

/* {{{ init_item() */

void ml_set_item( value item_type, value init_value )
{
    enum init_item { Width, Height, Frames_per_sec, Total_seconds };

    switch (Int_val(item_type))
    {
        case Width:    width  = Int_val(init_value); break;
        case Height:   height = Int_val(init_value); break;
        case Frames_per_sec:  frames_per_sec = Int_val(init_value); break;
        case Total_seconds:   total_seconds  = Int_val(init_value); break;
    }
    return;
}
/* }}} */

/* {{{ main */

int main(int argc, char **argv)
{
    // Initialize Caml code
    caml_startup(argv);

    // Open the display
    dpy = XOpenDisplay(NIL);
    assert(dpy);

    int blackColor = BlackPixel(dpy, DefaultScreen(dpy));
    int whiteColor = WhitePixel(dpy, DefaultScreen(dpy));

    width  = call_caml_init(0);
    height = call_caml_init(1);
    frames_per_sec = call_caml_init(2);
    total_seconds  = call_caml_init(3);

    // Create the window
    w = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy),
                            0, 0, width, height, 0, blackColor, blackColor);

    // We want to get MapNotify events
    XSelectInput(dpy, w, StructureNotifyMask);

    // "Map" the window (that is, make it appear on the screen)
    XMapWindow(dpy, w);

    // {{{ colormap 
#if 0
    // access the screen's standard color map
    Colormap colormap = DefaultColormap(dpy, DefaultScreen(dpy));
#endif
#if 1
    // allocating a new color map, works as follows: 
    // first, find the default visual for our screen.
    Visual* default_visual = DefaultVisual(dpy, DefaultScreen(dpy));
    // this creates a new color map. the number of color entries in this map
    // is determined by the number of colors supported on the given screen.
    colormap = XCreateColormap(dpy, w, default_visual, AllocNone);
#endif
    // }}}

    int i;
    // Create Foregrounds Graphics Contexts
    for (i=0; i < FOREGROUNDS; i++) {
        gc_foreground[i] = XCreateGC(dpy, w, 0, NIL);
        XSetForeground(dpy, gc_foreground[i], whiteColor);
    }

    gc_background = XCreateGC(dpy, w, 0, NIL);
    XSetForeground(dpy, gc_background, blackColor);

    // Wait for the MapNotify event
    for(;;) {
        XEvent e;
        XNextEvent(dpy, &e);
        if (e.type == MapNotify)
            break;
    }

    int j = frames_per_sec * total_seconds;
    for (i=0; i < j; i++)
    {
        call_caml_loop_iter(i);
        XFlush(dpy);  // Send the requests to the server
        usleep(1000000 / frames_per_sec);
    }

    sleep(3);
    return 0;
}
/* }}} */


/* vim: sw=4 sts=4 ts=4 et fdm=marker
 */
